import {Component, EventEmitter, Input, OnInit, Output, ViewChild} from '@angular/core';
import { Subscription } from "rxjs";
import {SimuInfoPresentation, Simulation} from "../../../../shared/entities/simulation";
import { Router} from "@angular/router";
import { NgxSmartModalService } from "ngx-smart-modal";
import { ApiService } from "../../../../shared/services/api.service";
import { ComboboxService } from "../../../../shared/services/combobox.service";
import { EntityService } from "../../../../shared/services/entity.service";
import { IgxGridComponent } from "@infragistics/igniteui-angular";
import {validateFromLoadJfcalc, validateFromPerfCalcs, validateFromSearchJfcalc} from "../../validateSimulation";
import {convertDataSearchJf} from "../../../../shared/utils/convert-data-search-jf";
import {changeNameItemInMenu} from "../../../../shared/utils/name-item-modal-search-jf";

@Component({
  selector: 'app-jf-calc-load-modal',
  templateUrl: './jf-calc-load-modal.component.html',
  styleUrls: ['./jf-calc-load-modal.component.scss']
})
export class JfCalcLoadModalComponent implements OnInit {
  subsc: Subscription = new Subscription();
  simulation: Simulation;

  @ViewChild('gridSearch', { static: true }) public gridSearch: IgxGridComponent;

  @Input() radiatorTypeName: null | string = null;

  @Input() countColumn: null | number = 16;

  @Output() loading = new EventEmitter<boolean>();

  @Output() updateRowdata = new EventEmitter<boolean>();

  combo1: any;
  combo10: any;

  searchCond = {
    JF_DATA_NAME: '',
    RADIATOR_TYPE: '',
    PROJECT_CD: '',
    HINBAN: '',
    KISHU: '',
    CORE_TYPE: '',
    IS_JF_CALCULATED: '1',
    PUBLISH_STATUS: '',
    FIN_PITCH_MM_A: '',
    FIN_PITCH_MM_B: '',
    CORE_COLUMN: '',
    JF_DATA_TYPE: '',
  };

  rowSearchData = [];

  cellCls = {
    'text-center': true,
  };

  isCoreColumnJf = false;
  isSideInnerFinPitchJf = false;
  isFinPitchJf = false;
  isFinPitchOuterJf = false;
  isFinPitchAJf = false;
  isFinPitchBJf = false;
  isInnerFinPitchJf = false;

  copyOptions = {
    enabled: true,
    copyHeaders: false,
    copyFormatters: false,
    separator: '\t'
  };


  selectedModal: string = 'op1';
  jfDataNames = [];
  searchValue: string = null;
  jfDataNameSelected: string = null;
  conditionFieldOp2 = {
    FIN_PITCH_MM_A: null,
    IN_CORE_WIDTH_MM: null,
    IN_CORE_HEIGHT_MM: null,
    FLOW_RATE_A: null,
    INLET_TEMP_A: null,
    FLOW_RATE_B: null,
    INLET_TEMP_B: null,
    RS_A: null,
    RS_B: null,
    PL_RS: null,
  };

  simuData: SimuInfoPresentation[] = [];

  constructor(private router: Router, public ngxSmartModalService: NgxSmartModalService, public api: ApiService, public combobox: ComboboxService, public entity: EntityService) {
    this.combobox.getData(['1', '10']).subscribe((res: any) => {
      this.combo1 = res.CMBMap['1'] ? res.CMBMap['1'] : [];
      this.combo10 = res.CMBMap['10'] ? res.CMBMap['10'] : [];
    });
  }

  ngOnInit(): void {
    this.subsc.add(this.entity.simulation$.subscribe(Simulation => {
      this.simulation = Simulation;

      this.setValueItemJF(changeNameItemInMenu(this.simulation.RADIATOR_TYPE));
    }));
  }

  ngOnDestroy(){
    this.subsc.unsubscribe();
  }

  searchJfcalc() {
    if (validateFromSearchJfcalc()) {
      this.loading.emit(true);
      this.searchCond.RADIATOR_TYPE = this.simulation.heatExchanger.type;

      this.simulation.searchJfcalc(this.searchCond, this.api).subscribe(res => {
        convertDataSearchJf(res.OutputData);
        this.rowSearchData = res.OutputData;
        this.loading.emit(false);
      }, err => {
        this.rowSearchData = [];
        this.loading.emit(false);
      });
    }
  }

  // loadJfcalc() {
  //   if (this.gridSearch.selectedRows) {
  //     const name = this.gridSearch.selectedRows[0];
  //
  //     if (validateFromLoadJfcalc(name)) {
  //       this.loading.emit(true);
  //       const currentSelectNo = this.api.simulationCondClipboad.currentSelectNo;
  //
  //       const param = {
  //         JF_DATA_NAME: name,
  //         KOUMOKU: currentSelectNo
  //       };
  //
  //       this.simulation.readJfdata(param, this.api, false).subscribe(res => {
  //         this.simulation.SimuInfo[currentSelectNo - 1] = res;
  //         if (!this.simulation.SIMULATE_DATA_NAME) {
  //           this.simulation.SimuInfoOriginal[currentSelectNo - 1] = this.simulation.setSimuInfoOriginal(this.simulation, res, currentSelectNo - 1);
  //         }
  //
  //         this.updateRowdata.emit(false);
  //         this.api.simulationCondClipboad.selectedColumn[currentSelectNo] = false;
  //         this.ngxSmartModalService.getModal('jfcalcLoadModal').close();
  //         this.rowSearchData = [];
  //         this.loading.emit(false);
  //       }, err => {
  //         this.loading.emit(false);
  //       });
  //     }
  //   }
  // }

  getCombo1Name(value) {
    const c = this.combo1.find(v => v.CODE == value);
    if (c) {
      return c.VALUE;
    }
    return '';
  }

  convertNum(num) {
    let result = null;

    if (num) {
      num = Number(num);
      result = (num * 10000) / 100;

      return `${result.toFixed(4)}%`;
    }

    return result;
  }

  setValueItemJF(arrIndex = []) {
    this.isCoreColumnJf = arrIndex[0];
    this.isFinPitchJf = arrIndex[1];
    this.isFinPitchAJf = arrIndex[2];
    this.isFinPitchBJf = arrIndex[3];
    this.isFinPitchOuterJf = arrIndex[4];
    this.isSideInnerFinPitchJf = arrIndex[5];
    this.isInnerFinPitchJf = arrIndex[6];
  }

  clickModalCalculation() {
    this.jfDataNames = this.gridSearch.selectedRows;
    this.jfDataNameSelected = null;

    if (this.jfDataNames.length > 0) {
      this.ngxSmartModalService.getModal('jfcalcLoadModal').close();
      this.ngxSmartModalService.getModal('modalCalculation').open();
      this.clearData();
    } else {
      alert('データなし');
    }
  }

  async clickCalculate() {
    this.loading.emit(true);

    if (this.selectedModal === 'op1') {
      for (const [index, jfDataName] of this.jfDataNames.entries()) {
        if (index < this.countColumn) {
          await this.readJfdata(index, jfDataName);
        }
      }

      await this.setSimulationOp1();
      await this.updateRowdata.emit(false);
      this.rowSearchData = [];
      this.jfDataNames = [];
      this.ngxSmartModalService.getModal('modalCalculation').close();
      this.loading.emit(false);
    } else {
      if (this.jfDataNameSelected) {
        await this.readJfdata(0, this.jfDataNameSelected);
        await this.setSimulationOp2();
        await this.updateRowdata.emit(false);
        this.perfCalcs();
      } else {
        alert('JF_DATA_NAMEが必須となります。')
        this.loading.emit(false);
      }
    }
  }

  clearData() {
    this.selectedModal = 'op1';
    this.simuData = [];

    for (const [key, value] of Object.entries(this.conditionFieldOp2)) {
      this.conditionFieldOp2[key] = null;
    }
  }

  setSimulationOp1() {
    this.simuData.map((v, i) => {
      const index = this.api.simulationCondClipboad.currentSelectNo - 1 + i;
      this.simulation.SimuInfo[index] = v;

      if (!this.simulation.SIMULATE_DATA_NAME) {
        this.simulation.SimuInfoOriginal[i] = this.simulation.setSimuInfoOriginal(this.simulation, v, i);
      }
    });
  }

  setSimulationOp2() {
    this.simulation.SimuInfo = [];
    this.simulation.SimuInfoOriginal = [];
    this.simulation.SimuInfo = this.simuData;

    if (!this.simulation.SIMULATE_DATA_NAME) {
      this.simulation.SimuInfo.map((v, i) => {
        this.simulation.SimuInfoOriginal[i] = this.simulation.setSimuInfoOriginal(this.simulation, v, i);
      })
    }
  }

  readJfdata(index, jfDataName) {
    return new Promise((resolve, reject) => {
      let param = {
        JF_DATA_NAME: jfDataName,
        KOUMOKU: this.api.simulationCondClipboad.currentSelectNo + index
      };

      if (this.selectedModal === 'op2') {
        param.KOUMOKU = index + 1;
      }

      this.simulation.readJfdata(param, this.api, false).subscribe(res => {
        this.simuData[index] = res;

        if (this.selectedModal === 'op2') {
          for (const [key, value] of Object.entries(this.conditionFieldOp2)) {
            this.simuData[index][key] = value;
          }
        }

        this.api.simulationCondClipboad.selectedColumn[index+1] = false;
        resolve(res);
      }, err => {
        console.log(err);
        this.loading.emit(false);
        reject('error');
      });
    });
  }

  perfCalcs() {
    if (validateFromPerfCalcs(this.simulation)) {
      this.simulation.perfCalcs(this.simulation, this.api).subscribe( res => {
        let perfCalcsResults = [];
        let index = 0;
        this.simulation.SimuInfo.map((v, i) => {
          if (v) {
            perfCalcsResults[i] = res.perfCalcsResults[index];
            index++;
          }
        });
        this.simulation.SimuInfoResult = {perfCalcsResults: perfCalcsResults};
        this.ngxSmartModalService.getModal('modalCalculation').close();
        this.router.navigate(['/heat-exchanger/simulation/result']);
      }, err => {
        console.log(err);
        this.loading.emit(false);
      }, () => {
        this.loading.emit(false);
      });
    }
  }

  selectJfDataName(jfDataName) {
    this.loading.emit(true);
    const param = {
      JF_DATA_NAME: jfDataName,
      KOUMOKU: 1,
    };

    this.simulation.readJfdata(param, this.api, false).subscribe(res => {
      for (const [key, value] of Object.entries(this.conditionFieldOp2)) {
        this.conditionFieldOp2[key] = res[key];
      }
      this.jfDataNameSelected = jfDataName;
    }, err => {
      console.log(err);
      this.loading.emit(false);
    }, () => {
      this.clearInput();
      this.loading.emit(false);
    });
  }

  clearInput() {
    this.searchValue = null;
  }

  onSelectionChanging(event) {
    if (event.newSelection.id === 'searchItem') {
      document.getElementById('search-value').focus();
      event.cancel = true;
    }
  }
}
